/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package container

import (
	corev1 "k8s.io/api/core/v1"
)

type Mutate interface {
	// Parameters
	GetVolumeMounts() []corev1.VolumeMount
	GetEnvFrom() []corev1.EnvFromSource
	GetEnvVar() []corev1.EnvVar
	// Runtime
	GetContainerName() string
	GetContainerImage() string
	GetContainerImagePullPolicy() corev1.PullPolicy
	GetContainerPorts() []corev1.ContainerPort
	GetContainerLivenessProbe() *corev1.Probe
	GetContainerReadinessProbe() *corev1.Probe
	GetContainerArgs() []string
	GetContainerCommand() []string
}

func MutateContainer(r Mutate, obj *corev1.Container) error {
	obj.Name = r.GetContainerName()
	obj.Image = r.GetContainerImage()
	obj.ImagePullPolicy = r.GetContainerImagePullPolicy()
	obj.Ports = r.GetContainerPorts()
	obj.LivenessProbe = r.GetContainerLivenessProbe()
	obj.ReadinessProbe = r.GetContainerReadinessProbe()
	obj.Args = r.GetContainerArgs()
	obj.Command = r.GetContainerCommand()

	obj.EnvFrom = r.GetEnvFrom()
	obj.Env = r.GetEnvVar()

	obj.VolumeMounts = r.GetVolumeMounts()

	return nil
}
